AUTH = '/auth'
SESSIONS = '/sessions'
USERS = '/users'
PODS = '/pods'
UNITS = '/units'
CHARGE_SCHEDULES = '/charge-schedules'
CHARGES = '/charges'

API_BASE = 'api.pod-point.com'
API_VERSION = 'v4'
API_BASE_URL = 'https://' + API_BASE + '/' + API_VERSION